@@ -52,6 +52,8 @@ SUPPORTED_CUDA_VERSIONS = SUPPORTED_CUDA_VERSIONS_ALMA8 + [x for x in SUPPORTED_
 
 CDE_BASE = "cde-base"
 CDE_PLUGINS = "cde-plugins"
+POD_DKUINSTALLDIR = "/opt/dataiku/installdir"
+POD_RO_DATA = "/home/dataiku-ro/data"
 
 # Keep values in sync with BaseImageBuilder.java
 class PythonInstallOption(Enum):
@@ -90,7 +92,7 @@ def get_node_type():
     try:
         with open(osp.join(DIP_HOME, "install.ini")) as f:
             config = configparser.RawConfigParser()
-            config.readfp(f)
+            config.read_file(f)
         return config.get('general', 'nodetype')
     except:
         logging.warning("Unable to find node type, assuming design node")
@@ -333,10 +335,16 @@ RUN dnf -y install python3.12-devel && dnf clean all
         if opts.py313:
             if self.opts.distrib == 'almalinux8':
                 raise Exception("Python 3.13 is not supported on AlmaLinux 8, please remove the --with-py313 option")
-            self.copy_to_buildenv(osp.join(DKUINSTALLDIR, "resources/container-exec/build-python313.sh"))
             self.dockerfile += """
-COPY build-python313.sh build/
-RUN build/build-python313.sh >/tmp/build-python.log && rm -f /tmp/build-python.log
+RUN dnf -y install python3.13-devel && dnf clean all
+"""
+        if opts.py314:
+            if self.opts.distrib == 'almalinux8':
+                raise Exception("Python 3.14 is not supported on AlmaLinux 8, please remove the --with-py314 option")
+            self.copy_to_buildenv(osp.join(DKUINSTALLDIR, "resources/container-exec/build-python314.sh"))
+            self.dockerfile += """
+COPY build-python314.sh build/
+RUN build/build-python314.sh >/tmp/build-python.log && rm -f /tmp/build-python.log
 """
 
     def add_r_base(self):
@@ -692,9 +700,26 @@ class KitImageBuilder(CompleteImageBuilder):
 
     # Comes in addition to DEFAULT_ALMA_SYSPACKAGES
     KIT_ALMA_SYSPACKAGES = [
-        "acl", "java-17-openjdk-headless", "dejavu-sans-fonts", "freetype", "fontconfig"
+        "acl",
+        "java-17-openjdk-headless",
+        "dejavu-sans-fonts",
+        "freetype",
+        "fontconfig",
     ]
 
+    def move_to_readonly_data(self, *subdirs):
+        self.dockerfile += """
+RUN mkdir -p {pod_ro_data}""".format(pod_ro_data=POD_RO_DATA)
+        for subdir in subdirs:
+            # create it in /home/dataiku (wo error) > move it to /home/dataiku-ro > link it back to /home/dataiku
+            self.dockerfile += """ && mkdir -p {pod_diphome}/{subdir} && mv {pod_diphome}/{subdir} {pod_ro_data}/ && ln -s {pod_ro_data}/{subdir} {pod_diphome}/{subdir}""".format(
+                pod_diphome="/home/dataiku/data",
+                pod_ro_data=POD_RO_DATA,
+                subdir=subdir
+            )
+        self.dockerfile += """ && chown -Rh dataiku:dataiku {pod_ro_data}
+""".format(pod_ro_data=POD_RO_DATA)
+
     def setup_dataiku_user(self):
         self.dockerfile += """
 MAINTAINER Dataiku <dss@dataiku.com>
@@ -708,6 +733,11 @@ RUN groupadd -r dataiku \
 """.format(uid=get_dataiku_user_uid())
 
     def copy_kit(self):
+        # setup pod install dir
+        self.dockerfile += """
+RUN mkdir -p {podinstalldir}
+""".format(podinstalldir=POD_DKUINSTALLDIR)
+
         # see make-studio-package.sh
         if PYTHONVERSION == "3.9":
             PYTHON_PACKAGES = "python39.packages"
@@ -756,7 +786,7 @@ RUN groupadd -r dataiku \
             os.makedirs(osp.join(self.build_env_path, 'lib', 'ivy'))
             for ivy_conf in ["common-run", "backend-run"]:
                 self.copy_to_buildenv(osp.join(DKUINSTALLDIR, 'lib', 'ivy', ivy_conf), osp.join("lib", "ivy", ivy_conf))
-            for lib_conf in ["third", "shadelib", "shims"]:
+            for lib_conf in ["third", "shadelib", "dss-shadelib", "shims"]:
                 self.copy_to_buildenv(osp.join(DKUINSTALLDIR, 'lib', lib_conf), osp.join("lib", lib_conf))
             for ivy_conf in glob.glob(osp.join(DKUINSTALLDIR, 'lib', 'ivy', 'jdbc*')):
                 self.copy_to_buildenv(ivy_conf, osp.join("lib", "ivy", osp.basename(ivy_conf)))
@@ -807,12 +837,12 @@ RUN groupadd -r dataiku \
             # then we can copy all of that in the container
             for item in ["dist", "dss-version.json", "dss-arch.txt", "installer.sh",
                          "lib", "python", PYTHON_PACKAGES, "R", "resources", "scripts", "conda.packages"]:
-                self.dockerfile += 'COPY "%s" /home/dataiku/installdir/%s\n' % (item, item)
+                self.dockerfile += 'COPY "%s" %s/%s\n' % (item, POD_DKUINSTALLDIR, item)
 
         else:
             for item in ["dist", "dss-version.json", "dss-arch.txt", "installer.sh",
                          "lib", "python", PYTHON_PACKAGES, "R", "resources", "scripts", "conda.packages"]:
-                self.dockerfile += 'COPY "%s" /home/dataiku/installdir/%s\n' % (item, item)
+                self.dockerfile += 'COPY "%s" %s/%s\n' % (item, POD_DKUINSTALLDIR, item)
                 self.copy_to_buildenv(osp.join(DKUINSTALLDIR, item), item)
 
             # Remove some files that are not actually useful and that trigger stupid security scanners
@@ -822,12 +852,13 @@ RUN groupadd -r dataiku \
             # xmlsec/xmltooling/opensaml/esapi/not-yet-commons: used by SAML login, not used in API node
             # nbconvert: not needed in API node
             self.dockerfile += """
-RUN rm -f /home/dataiku/installdir/resources/code-envs/python/image-classification/v1/spec/requirements.txt \
-    && rm -f /home/dataiku/installdir/resources/code-envs/python/object-detection/v1/spec/requirements.txt \
-    && rm -rf /home/dataiku/installdir/lib/ivy/govern-server-run \
-    && rm -f /home/dataiku/installdir/lib/ivy/backend-run/jnr-posix-3.0.44.jar \
-    && rm -rf /home/dataiku/installdir/python36.packages/nbconvert*
-"""
+RUN rm -f {podinstalldir}/resources/code-envs/python/image-classification/v1/spec/requirements.txt \
+    && rm -f {podinstalldir}/resources/code-envs/python/object-detection/v1/spec/requirements.txt \
+    && rm -rf {podinstalldir}/lib/ivy/govern-server-run \
+    && rm -f {podinstalldir}/lib/ivy/backend-run/jnr-posix-3.0.44.jar \
+    && rm -rf {podinstalldir}/python36.packages/nbconvert* \
+    && chown -Rh dataiku:dataiku {podinstalldir}
+""".format(podinstalldir=POD_DKUINSTALLDIR)
 
 class APIImageBuilder(KitImageBuilder):
     """
@@ -856,8 +887,9 @@ class APIImageBuilder(KitImageBuilder):
         self.copy_kit()
 
         self.dockerfile += """
-RUN /bin/su - dataiku -c "/home/dataiku/installdir/installer.sh -y -t api -d /home/dataiku/data -p 12000 -P python%s"
-""" % PYTHONVERSION
+RUN /bin/su - dataiku -c "%s/installer.sh -y -t api -d /home/dataiku/data -p 12000 -P python%s"
+""" % (POD_DKUINSTALLDIR, PYTHONVERSION)
+        self.move_to_readonly_data("lib", "plugins", "R.lib", "pyenv")
         if self.opts.r:
             repo_option = "-repo %s" % self.opts.cran_mirror if self.opts.cran_mirror else ""
             self.dockerfile += """
@@ -898,8 +930,12 @@ class CDEImageBuilder(KitImageBuilder):
         self.copy_kit()
 
         self.dockerfile += """
-RUN /bin/su - dataiku -c "/home/dataiku/installdir/installer.sh -y -t design -d /home/dataiku/data -p 12000 -P python%s -j"
-""" % PYTHONVERSION
+RUN /bin/su - dataiku -c "%s/installer.sh -y -t design -d /home/dataiku/data -p 12000 -P python%s -j"
+""" % (POD_DKUINSTALLDIR, PYTHONVERSION)
+
+        # move outside /home/dataiku some dirs to speedup the initcontainer for readOnlyRootFilesystem
+        self.move_to_readonly_data("lib", "plugins", "R.lib", "pyenv")
+
         if self.opts.r:
             repo_option = "-repo %s" % self.opts.cran_mirror if self.opts.cran_mirror else ""
             self.dockerfile += 'RUN /bin/su - dataiku -c "/home/dataiku/data/bin/dssadmin install-R-integration -noDeps %s -noJupyterKernel"' % repo_option
@@ -918,9 +954,9 @@ RUN /bin/su - dataiku -c "/home/dataiku/installdir/installer.sh -y -t design -d
         if hadoop_standalone_libs_folder is not None:
             self.copy_to_buildenv(hadoop_standalone_libs_folder, 'hadoop-standalone-libs')
             self.dockerfile += """
-COPY hadoop-standalone-libs /home/dataiku/installdir/hadoop-standalone-libs/
-RUN /home/dataiku/data/pyenv/bin/python /home/dataiku/installdir/scripts/_build_standalone_hadoop_env.py /home/dataiku/data/bin/env-hadoop.sh
-"""
+COPY hadoop-standalone-libs {podinstalldir}/hadoop-standalone-libs/
+RUN /home/dataiku/data/pyenv/bin/python {podinstalldir}/scripts/_build_standalone_hadoop_env.py /home/dataiku/data/bin/env-hadoop.sh
+""".format(podinstalldir=POD_DKUINSTALLDIR)
 
         self.dockerfile += """
 WORKDIR /opt/dataiku
@@ -1096,7 +1132,7 @@ ENTRYPOINT ["/home/dataiku/data/bin/cde"]
             if needs_conda:
                 # Miniconda.Dockerfile have a similar chunk, have a look there if you update here
                 self.dockerfile += """
-RUN if ! test -d /opt/conda; then curl -sSL https://repo.anaconda.com/miniconda/Miniconda3-py37_23.1.0-1-Linux-x86_64.sh -o /tmp/miniconda.sh \
+RUN if ! test -d /opt/conda; then curl -sSL https://repo.anaconda.com/miniconda/Miniconda3-py39_25.1.1-2-Linux-x86_64.sh -o /tmp/miniconda.sh \
     && bash /tmp/miniconda.sh -bfp /opt/conda/ \
     && rm -rf /tmp/miniconda.sh; fi
 ENV PATH="/opt/conda/bin:$PATH"
@@ -1287,6 +1323,9 @@ if __name__ == '__main__':
     parser.add_argument('--with-py313',action="store_true", dest="py313", help="Enable Python 3.13, supported on AlmaLinux 9 only (default: disabled)")
     parser.add_argument('--without-py313',action="store_false", dest="py313", help="Disable Python 3.13 (default: disabled)")
 
+    parser.add_argument('--with-py314', action="store_true", dest="py314", help="Enable Python 3.14, supported on AlmaLinux 9 only (default: disabled)")
+    parser.add_argument('--without-py314', action="store_false", dest="py314", help="Disable Python 3.14 (default: disabled)")
+
     parser.add_argument('--with-cuda', action="store_true", dest="cuda", help="Enable CUDA (default: disabled)")
     parser.add_argument('--without-cuda',action="store_false", dest="cuda", help="Disable CUDA (default: disabled)")
     parser.add_argument('--cuda-version', type=str, choices=SUPPORTED_CUDA_VERSIONS, help="CUDA version (default: " + DEFAULT_CUDA_VERSION_ALMA9 + " on AlmaLinux 9, " + DEFAULT_CUDA_VERSION_ALMA8 + " on AlmaLinux 8)")
